#ifndef vtk_haru_mangle_h
#define vtk_haru_mangle_h

#define ConvertDateToXMDate vtklibharu_ConvertDateToXMDate
#define ExtGState_Check vtklibharu_ExtGState_Check
#define GetElement vtklibharu_GetElement
#define HPDF_3DAnnotExData_New vtklibharu_HPDF_3DAnnotExData_New
#define HPDF_3DAnnotExData_Set3DMeasurement vtklibharu_HPDF_3DAnnotExData_Set3DMeasurement
#define HPDF_3DAnnot_New vtklibharu_HPDF_3DAnnot_New
#define HPDF_3DAnnot_Set3DView vtklibharu_HPDF_3DAnnot_Set3DView
#define HPDF_3DC3DMeasure_New vtklibharu_HPDF_3DC3DMeasure_New
#define HPDF_3DC3DMeasure_SetProjectionAnotation vtklibharu_HPDF_3DC3DMeasure_SetProjectionAnotation
#define HPDF_3DC3DMeasure_SetText vtklibharu_HPDF_3DC3DMeasure_SetText
#define HPDF_3DC3DMeasure_SetTextBoxSize vtklibharu_HPDF_3DC3DMeasure_SetTextBoxSize
#define HPDF_3DMeasure_SetColor vtklibharu_HPDF_3DMeasure_SetColor
#define HPDF_3DMeasure_SetName vtklibharu_HPDF_3DMeasure_SetName
#define HPDF_3DMeasure_SetTextSize vtklibharu_HPDF_3DMeasure_SetTextSize
#define HPDF_3DView_Add3DC3DMeasure vtklibharu_HPDF_3DView_Add3DC3DMeasure
#define HPDF_3DView_AddNode vtklibharu_HPDF_3DView_AddNode
#define HPDF_3DView_New vtklibharu_HPDF_3DView_New
#define HPDF_3DView_SetBackgroundColor vtklibharu_HPDF_3DView_SetBackgroundColor
#define HPDF_3DView_SetCamera vtklibharu_HPDF_3DView_SetCamera
#define HPDF_3DView_SetLighting vtklibharu_HPDF_3DView_SetLighting
#define HPDF_3DView_SetOrthogonalProjection vtklibharu_HPDF_3DView_SetOrthogonalProjection
#define HPDF_3DView_SetPerspectiveProjection vtklibharu_HPDF_3DView_SetPerspectiveProjection
#define HPDF_AddColorspaceFromProfile vtklibharu_HPDF_AddColorspaceFromProfile
#define HPDF_AddIntent vtklibharu_HPDF_AddIntent
#define HPDF_AddPage vtklibharu_HPDF_AddPage
#define HPDF_AddPageLabel vtklibharu_HPDF_AddPageLabel
#define HPDF_Annotation_New vtklibharu_HPDF_Annotation_New
#define HPDF_Annotation_SetBorderStyle vtklibharu_HPDF_Annotation_SetBorderStyle
#define HPDF_Annotation_Validate vtklibharu_HPDF_Annotation_Validate
#define HPDF_Annot_Set3DView vtklibharu_HPDF_Annot_Set3DView
#define HPDF_Annot_SetCMYKColor vtklibharu_HPDF_Annot_SetCMYKColor
#define HPDF_Annot_SetGrayColor vtklibharu_HPDF_Annot_SetGrayColor
#define HPDF_Annot_SetNoColor vtklibharu_HPDF_Annot_SetNoColor
#define HPDF_Annot_SetRGBColor vtklibharu_HPDF_Annot_SetRGBColor
#define HPDF_Array_Add vtklibharu_HPDF_Array_Add
#define HPDF_Array_AddName vtklibharu_HPDF_Array_AddName
#define HPDF_Array_AddNumber vtklibharu_HPDF_Array_AddNumber
#define HPDF_Array_AddReal vtklibharu_HPDF_Array_AddReal
#define HPDF_Array_Clear vtklibharu_HPDF_Array_Clear
#define HPDF_Array_Free vtklibharu_HPDF_Array_Free
#define HPDF_Array_GetItem vtklibharu_HPDF_Array_GetItem
#define HPDF_Array_Insert vtklibharu_HPDF_Array_Insert
#define HPDF_Array_Items vtklibharu_HPDF_Array_Items
#define HPDF_Array_New vtklibharu_HPDF_Array_New
#define HPDF_Array_Write vtklibharu_HPDF_Array_Write
#define HPDF_AToF vtklibharu_HPDF_AToF
#define HPDF_AToI vtklibharu_HPDF_AToI
#define HPDF_AttachFile vtklibharu_HPDF_AttachFile
#define HPDF_Base14FontDef_FindBuiltinData vtklibharu_HPDF_Base14FontDef_FindBuiltinData
#define HPDF_Base14FontDef_New vtklibharu_HPDF_Base14FontDef_New
#define HPDF_BasicEncoder_CopyMap vtklibharu_HPDF_BasicEncoder_CopyMap
#define HPDF_BasicEncoder_FindBuiltinData vtklibharu_HPDF_BasicEncoder_FindBuiltinData
#define HPDF_BasicEncoder_Free vtklibharu_HPDF_BasicEncoder_Free
#define HPDF_BasicEncoder_New vtklibharu_HPDF_BasicEncoder_New
#define HPDF_BasicEncoder_OverrideMap vtklibharu_HPDF_BasicEncoder_OverrideMap
#define HPDF_BasicEncoder_ToUnicode vtklibharu_HPDF_BasicEncoder_ToUnicode
#define HPDF_BasicEncoder_Write vtklibharu_HPDF_BasicEncoder_Write
#define HPDF_Binary_Free vtklibharu_HPDF_Binary_Free
#define HPDF_Binary_GetLen vtklibharu_HPDF_Binary_GetLen
#define HPDF_Binary_GetValue vtklibharu_HPDF_Binary_GetValue
#define HPDF_Binary_New vtklibharu_HPDF_Binary_New
#define HPDF_Binary_SetValue vtklibharu_HPDF_Binary_SetValue
#define HPDF_Binary_Write vtklibharu_HPDF_Binary_Write
#define HPDF_Boolean_New vtklibharu_HPDF_Boolean_New
#define HPDF_Boolean_Write vtklibharu_HPDF_Boolean_Write
#define HPDF_Box_Array_New vtklibharu_HPDF_Box_Array_New
#define HPDF_CallbackReader_New vtklibharu_HPDF_CallbackReader_New
#define HPDF_CallbackWriter_New vtklibharu_HPDF_CallbackWriter_New
#define HPDF_Catalog_AddPageLabel vtklibharu_HPDF_Catalog_AddPageLabel
#define HPDF_Catalog_GetNames vtklibharu_HPDF_Catalog_GetNames
#define HPDF_Catalog_GetPageLayout vtklibharu_HPDF_Catalog_GetPageLayout
#define HPDF_Catalog_GetPageMode vtklibharu_HPDF_Catalog_GetPageMode
#define HPDF_Catalog_GetRoot vtklibharu_HPDF_Catalog_GetRoot
#define HPDF_Catalog_GetViewerPreference vtklibharu_HPDF_Catalog_GetViewerPreference
#define HPDF_Catalog_New vtklibharu_HPDF_Catalog_New
#define HPDF_Catalog_SetNames vtklibharu_HPDF_Catalog_SetNames
#define HPDF_Catalog_SetOpenAction vtklibharu_HPDF_Catalog_SetOpenAction
#define HPDF_Catalog_SetPageLayout vtklibharu_HPDF_Catalog_SetPageLayout
#define HPDF_Catalog_SetPageMode vtklibharu_HPDF_Catalog_SetPageMode
#define HPDF_Catalog_SetViewerPreference vtklibharu_HPDF_Catalog_SetViewerPreference
#define HPDF_Catalog_Validate vtklibharu_HPDF_Catalog_Validate
#define HPDF_CheckError vtklibharu_HPDF_CheckError
#define HPDF_CIDFontDef_AddWidth vtklibharu_HPDF_CIDFontDef_AddWidth
#define HPDF_CIDFontDef_ChangeStyle vtklibharu_HPDF_CIDFontDef_ChangeStyle
#define HPDF_CIDFontDef_FreeFunc vtklibharu_HPDF_CIDFontDef_FreeFunc
#define HPDF_CIDFontDef_FreeWidth vtklibharu_HPDF_CIDFontDef_FreeWidth
#define HPDF_CIDFontDef_GetCIDWidth vtklibharu_HPDF_CIDFontDef_GetCIDWidth
#define HPDF_CIDFontDef_New vtklibharu_HPDF_CIDFontDef_New
#define HPDF_CMapEncoder_AddCMap vtklibharu_HPDF_CMapEncoder_AddCMap
#define HPDF_CMapEncoder_AddCodeSpaceRange vtklibharu_HPDF_CMapEncoder_AddCodeSpaceRange
#define HPDF_CMapEncoder_AddJWWLineHead vtklibharu_HPDF_CMapEncoder_AddJWWLineHead
#define HPDF_CMapEncoder_AddNotDefRange vtklibharu_HPDF_CMapEncoder_AddNotDefRange
#define HPDF_CMapEncoder_ByteType vtklibharu_HPDF_CMapEncoder_ByteType
#define HPDF_CMapEncoder_Free vtklibharu_HPDF_CMapEncoder_Free
#define HPDF_CMapEncoder_InitAttr vtklibharu_HPDF_CMapEncoder_InitAttr
#define HPDF_CMapEncoder_New vtklibharu_HPDF_CMapEncoder_New
#define HPDF_CMapEncoder_SetUnicodeArray vtklibharu_HPDF_CMapEncoder_SetUnicodeArray
#define HPDF_CMapEncoder_ToCID vtklibharu_HPDF_CMapEncoder_ToCID
#define HPDF_CMapEncoder_ToUnicode vtklibharu_HPDF_CMapEncoder_ToUnicode
#define HPDF_CMapEncoder_Write vtklibharu_HPDF_CMapEncoder_Write
#define HPDF_CopyError vtklibharu_HPDF_CopyError
#define HPDF_Create3DView vtklibharu_HPDF_Create3DView
#define HPDF_CreateExtGState vtklibharu_HPDF_CreateExtGState
#define HPDF_CreateJavaScript vtklibharu_HPDF_CreateJavaScript
#define HPDF_CreateOutline vtklibharu_HPDF_CreateOutline
#define HPDF_Destination_New vtklibharu_HPDF_Destination_New
#define HPDF_Destination_SetFit vtklibharu_HPDF_Destination_SetFit
#define HPDF_Destination_SetFitB vtklibharu_HPDF_Destination_SetFitB
#define HPDF_Destination_SetFitBH vtklibharu_HPDF_Destination_SetFitBH
#define HPDF_Destination_SetFitBV vtklibharu_HPDF_Destination_SetFitBV
#define HPDF_Destination_SetFitH vtklibharu_HPDF_Destination_SetFitH
#define HPDF_Destination_SetFitR vtklibharu_HPDF_Destination_SetFitR
#define HPDF_Destination_SetFitV vtklibharu_HPDF_Destination_SetFitV
#define HPDF_Destination_SetXYZ vtklibharu_HPDF_Destination_SetXYZ
#define HPDF_DESTINATION_TYPE_NAMES vtklibharu_HPDF_DESTINATION_TYPE_NAMES
#define HPDF_Destination_Validate vtklibharu_HPDF_Destination_Validate
#define HPDF_Dict_Add vtklibharu_HPDF_Dict_Add
#define HPDF_Dict_AddBoolean vtklibharu_HPDF_Dict_AddBoolean
#define HPDF_Dict_Add_FilterParams vtklibharu_HPDF_Dict_Add_FilterParams
#define HPDF_Dict_AddName vtklibharu_HPDF_Dict_AddName
#define HPDF_Dict_AddNumber vtklibharu_HPDF_Dict_AddNumber
#define HPDF_Dict_AddPoint3D vtklibharu_HPDF_Dict_AddPoint3D
#define HPDF_Dict_AddReal vtklibharu_HPDF_Dict_AddReal
#define HPDF_Dict_Free vtklibharu_HPDF_Dict_Free
#define HPDF_Dict_GetItem vtklibharu_HPDF_Dict_GetItem
#define HPDF_Dict_GetKeyByObj vtklibharu_HPDF_Dict_GetKeyByObj
#define HPDF_Dict_New vtklibharu_HPDF_Dict_New
#define HPDF_Dict_RemoveElement vtklibharu_HPDF_Dict_RemoveElement
#define HPDF_DictStream_New vtklibharu_HPDF_DictStream_New
#define HPDF_Dict_Write vtklibharu_HPDF_Dict_Write
#define HPDF_Doc_AddPagesTo vtklibharu_HPDF_Doc_AddPagesTo
#define HPDF_Doc_FindEncoder vtklibharu_HPDF_Doc_FindEncoder
#define HPDF_Doc_FindFont vtklibharu_HPDF_Doc_FindFont
#define HPDF_Doc_FindFontDef vtklibharu_HPDF_Doc_FindFontDef
#define HPDF_Doc_GetCurrentPages vtklibharu_HPDF_Doc_GetCurrentPages
#define HPDF_Doc_PrepareEncryption vtklibharu_HPDF_Doc_PrepareEncryption
#define HPDF_Doc_RegisterEncoder vtklibharu_HPDF_Doc_RegisterEncoder
#define HPDF_Doc_RegisterFontDef vtklibharu_HPDF_Doc_RegisterFontDef
#define HPDF_Doc_SetCurrentPage vtklibharu_HPDF_Doc_SetCurrentPage
#define HPDF_Doc_SetCurrentPages vtklibharu_HPDF_Doc_SetCurrentPages
#define HPDF_Doc_SetEncryptOff vtklibharu_HPDF_Doc_SetEncryptOff
#define HPDF_Doc_SetEncryptOn vtklibharu_HPDF_Doc_SetEncryptOn
#define HPDF_Doc_Validate vtklibharu_HPDF_Doc_Validate
#define HPDF_EmbeddedFile_New vtklibharu_HPDF_EmbeddedFile_New
#define HPDF_EmbeddedFile_Validate vtklibharu_HPDF_EmbeddedFile_Validate
#define HPDF_Encoder_ByteType vtklibharu_HPDF_Encoder_ByteType
#define HPDF_Encoder_CheckJWWLineHead vtklibharu_HPDF_Encoder_CheckJWWLineHead
#define HPDF_Encoder_Free vtklibharu_HPDF_Encoder_Free
#define HPDF_Encoder_GetByteType vtklibharu_HPDF_Encoder_GetByteType
#define HPDF_Encoder_GetType vtklibharu_HPDF_Encoder_GetType
#define HPDF_Encoder_GetUnicode vtklibharu_HPDF_Encoder_GetUnicode
#define HPDF_Encoder_GetWritingMode vtklibharu_HPDF_Encoder_GetWritingMode
#define HPDF_Encoder_SetParseText vtklibharu_HPDF_Encoder_SetParseText
#define HPDF_Encoder_ToUnicode vtklibharu_HPDF_Encoder_ToUnicode
#define HPDF_Encoder_Validate vtklibharu_HPDF_Encoder_Validate
#define HPDF_Encrypt_CreateEncryptionKey vtklibharu_HPDF_Encrypt_CreateEncryptionKey
#define HPDF_Encrypt_CreateOwnerKey vtklibharu_HPDF_Encrypt_CreateOwnerKey
#define HPDF_Encrypt_CreateUserKey vtklibharu_HPDF_Encrypt_CreateUserKey
#define HPDF_Encrypt_CryptBuf vtklibharu_HPDF_Encrypt_CryptBuf
#define HPDF_EncryptDict_CreateID vtklibharu_HPDF_EncryptDict_CreateID
#define HPDF_EncryptDict_GetAttr vtklibharu_HPDF_EncryptDict_GetAttr
#define HPDF_EncryptDict_New vtklibharu_HPDF_EncryptDict_New
#define HPDF_EncryptDict_OnFree vtklibharu_HPDF_EncryptDict_OnFree
#define HPDF_EncryptDict_Prepare vtklibharu_HPDF_EncryptDict_Prepare
#define HPDF_EncryptDict_SetPassword vtklibharu_HPDF_EncryptDict_SetPassword
#define HPDF_EncryptDict_Validate vtklibharu_HPDF_EncryptDict_Validate
#define HPDF_Encrypt_Init vtklibharu_HPDF_Encrypt_Init
#define HPDF_Encrypt_InitKey vtklibharu_HPDF_Encrypt_InitKey
#define HPDF_Encrypt_Reset vtklibharu_HPDF_Encrypt_Reset
#define HPDF_Error_GetCode vtklibharu_HPDF_Error_GetCode
#define HPDF_Error_GetDetailCode vtklibharu_HPDF_Error_GetDetailCode
#define HPDF_Error_Init vtklibharu_HPDF_Error_Init
#define HPDF_Error_Reset vtklibharu_HPDF_Error_Reset
#define HPDF_ExtGState_New vtklibharu_HPDF_ExtGState_New
#define HPDF_ExtGState_SetAlphaFill vtklibharu_HPDF_ExtGState_SetAlphaFill
#define HPDF_ExtGState_SetAlphaStroke vtklibharu_HPDF_ExtGState_SetAlphaStroke
#define HPDF_ExtGState_SetBlendMode vtklibharu_HPDF_ExtGState_SetBlendMode
#define HPDF_ExtGState_Validate vtklibharu_HPDF_ExtGState_Validate
#define HPDF_FileReader_New vtklibharu_HPDF_FileReader_New
#define HPDF_FileReader_ReadFunc vtklibharu_HPDF_FileReader_ReadFunc
#define HPDF_FileReader_SeekFunc vtklibharu_HPDF_FileReader_SeekFunc
#define HPDF_FileStream_FreeFunc vtklibharu_HPDF_FileStream_FreeFunc
#define HPDF_FileStream_SizeFunc vtklibharu_HPDF_FileStream_SizeFunc
#define HPDF_FileStream_TellFunc vtklibharu_HPDF_FileStream_TellFunc
#define HPDF_FileWriter_New vtklibharu_HPDF_FileWriter_New
#define HPDF_FileWriter_WriteFunc vtklibharu_HPDF_FileWriter_WriteFunc
#define HPDF_FontDef_Cleanup vtklibharu_HPDF_FontDef_Cleanup
#define HPDF_FontDef_Free vtklibharu_HPDF_FontDef_Free
#define HPDF_FontDef_Validate vtklibharu_HPDF_FontDef_Validate
#define HPDF_Font_GetAscent vtklibharu_HPDF_Font_GetAscent
#define HPDF_Font_GetBBox vtklibharu_HPDF_Font_GetBBox
#define HPDF_Font_GetCapHeight vtklibharu_HPDF_Font_GetCapHeight
#define HPDF_Font_GetDescent vtklibharu_HPDF_Font_GetDescent
#define HPDF_Font_GetEncodingName vtklibharu_HPDF_Font_GetEncodingName
#define HPDF_Font_GetFontName vtklibharu_HPDF_Font_GetFontName
#define HPDF_Font_GetUnicodeWidth vtklibharu_HPDF_Font_GetUnicodeWidth
#define HPDF_Font_GetXHeight vtklibharu_HPDF_Font_GetXHeight
#define HPDF_Font_MeasureText vtklibharu_HPDF_Font_MeasureText
#define HPDF_Font_TextWidth vtklibharu_HPDF_Font_TextWidth
#define HPDF_Font_Validate vtklibharu_HPDF_Font_Validate
#define HPDF_Free vtklibharu_HPDF_Free
#define HPDF_FreeDoc vtklibharu_HPDF_FreeDoc
#define HPDF_FreeDocAll vtklibharu_HPDF_FreeDocAll
#define HPDF_FreeMem vtklibharu_HPDF_FreeMem
#define HPDF_FreeTextAnnot_Set2PointCalloutLine vtklibharu_HPDF_FreeTextAnnot_Set2PointCalloutLine
#define HPDF_FreeTextAnnot_Set3PointCalloutLine vtklibharu_HPDF_FreeTextAnnot_Set3PointCalloutLine
#define HPDF_FreeTextAnnot_SetDefaultStyle vtklibharu_HPDF_FreeTextAnnot_SetDefaultStyle
#define HPDF_FreeTextAnnot_SetLineEndingStyle vtklibharu_HPDF_FreeTextAnnot_SetLineEndingStyle
#define HPDF_FToA vtklibharu_HPDF_FToA
#define HPDF_GetContents vtklibharu_HPDF_GetContents
#define HPDF_GetCurrentEncoder vtklibharu_HPDF_GetCurrentEncoder
#define HPDF_GetCurrentPage vtklibharu_HPDF_GetCurrentPage
#define HPDF_GetEncoder vtklibharu_HPDF_GetEncoder
#define HPDF_GetError vtklibharu_HPDF_GetError
#define HPDF_GetErrorDetail vtklibharu_HPDF_GetErrorDetail
#define HPDF_GetFont vtklibharu_HPDF_GetFont
#define HPDF_GetFontDef vtklibharu_HPDF_GetFontDef
#define HPDF_GetInfoAttr vtklibharu_HPDF_GetInfoAttr
#define HPDF_GetMem vtklibharu_HPDF_GetMem
#define HPDF_GetPageByIndex vtklibharu_HPDF_GetPageByIndex
#define HPDF_GetPageLayout vtklibharu_HPDF_GetPageLayout
#define HPDF_GetPageMode vtklibharu_HPDF_GetPageMode
#define HPDF_GetStreamSize vtklibharu_HPDF_GetStreamSize
#define HPDF_GetTTFontDefFromFile vtklibharu_HPDF_GetTTFontDefFromFile
#define HPDF_GetVersion vtklibharu_HPDF_GetVersion
#define HPDF_GetViewerPreference vtklibharu_HPDF_GetViewerPreference
#define HPDF_GryphNameToUnicode vtklibharu_HPDF_GryphNameToUnicode
#define HPDF_GState_Free vtklibharu_HPDF_GState_Free
#define HPDF_GState_New vtklibharu_HPDF_GState_New
#define HPDF_HasDoc vtklibharu_HPDF_HasDoc
#define HPDF_ICC_LoadIccFromMem vtklibharu_HPDF_ICC_LoadIccFromMem
#define HPDF_Image_AddSMask vtklibharu_HPDF_Image_AddSMask
#define HPDF_Image_GetBitsPerComponent vtklibharu_HPDF_Image_GetBitsPerComponent
#define HPDF_Image_GetColorSpace vtklibharu_HPDF_Image_GetColorSpace
#define HPDF_Image_GetHeight vtklibharu_HPDF_Image_GetHeight
#define HPDF_Image_GetSize vtklibharu_HPDF_Image_GetSize
#define HPDF_Image_GetSize2 vtklibharu_HPDF_Image_GetSize2
#define HPDF_Image_GetWidth vtklibharu_HPDF_Image_GetWidth
#define HPDF_Image_Load1BitImageFromMem vtklibharu_HPDF_Image_Load1BitImageFromMem
#define HPDF_Image_LoadJpegImage vtklibharu_HPDF_Image_LoadJpegImage
#define HPDF_Image_LoadJpegImageFromMem vtklibharu_HPDF_Image_LoadJpegImageFromMem
#define HPDF_Image_LoadPngImage vtklibharu_HPDF_Image_LoadPngImage
#define HPDF_Image_LoadRaw1BitImageFromMem vtklibharu_HPDF_Image_LoadRaw1BitImageFromMem
#define HPDF_Image_LoadRawImage vtklibharu_HPDF_Image_LoadRawImage
#define HPDF_Image_LoadRawImageFromMem vtklibharu_HPDF_Image_LoadRawImageFromMem
#define HPDF_Image_SetColorMask vtklibharu_HPDF_Image_SetColorMask
#define HPDF_Image_SetColorSpace vtklibharu_HPDF_Image_SetColorSpace
#define HPDF_Image_SetMask vtklibharu_HPDF_Image_SetMask
#define HPDF_Image_SetMaskImage vtklibharu_HPDF_Image_SetMaskImage
#define HPDF_Image_SetRenderingIntent vtklibharu_HPDF_Image_SetRenderingIntent
#define HPDF_Image_Validate vtklibharu_HPDF_Image_Validate
#define HPDF_Info_GetInfoAttr vtklibharu_HPDF_Info_GetInfoAttr
#define HPDF_Info_SetInfoAttr vtklibharu_HPDF_Info_SetInfoAttr
#define HPDF_Info_SetInfoDateAttr vtklibharu_HPDF_Info_SetInfoDateAttr
#define HPDF_InsertPage vtklibharu_HPDF_InsertPage
#define HPDF_IToA vtklibharu_HPDF_IToA
#define HPDF_IToA2 vtklibharu_HPDF_IToA2
#define HPDF_LineAnnot_SetCaption vtklibharu_HPDF_LineAnnot_SetCaption
#define HPDF_LineAnnot_SetLeader vtklibharu_HPDF_LineAnnot_SetLeader
#define HPDF_LineAnnot_SetPosition vtklibharu_HPDF_LineAnnot_SetPosition
#define HPDF_LinkAnnot_New vtklibharu_HPDF_LinkAnnot_New
#define HPDF_LinkAnnot_SetBorderStyle vtklibharu_HPDF_LinkAnnot_SetBorderStyle
#define HPDF_LinkAnnot_SetHighlightMode vtklibharu_HPDF_LinkAnnot_SetHighlightMode
#define HPDF_List_Add vtklibharu_HPDF_List_Add
#define HPDF_List_Clear vtklibharu_HPDF_List_Clear
#define HPDF_List_Find vtklibharu_HPDF_List_Find
#define HPDF_List_Free vtklibharu_HPDF_List_Free
#define HPDF_List_Insert vtklibharu_HPDF_List_Insert
#define HPDF_List_ItemAt vtklibharu_HPDF_List_ItemAt
#define HPDF_List_New vtklibharu_HPDF_List_New
#define HPDF_List_Remove vtklibharu_HPDF_List_Remove
#define HPDF_List_RemoveByIndex vtklibharu_HPDF_List_RemoveByIndex
#define HPDF_LoadIccProfileFromFile vtklibharu_HPDF_LoadIccProfileFromFile
#define HPDF_LoadJpegImageFromFile vtklibharu_HPDF_LoadJpegImageFromFile
#define HPDF_LoadJpegImageFromMem vtklibharu_HPDF_LoadJpegImageFromMem
#define HPDF_LoadPngImageFromFile vtklibharu_HPDF_LoadPngImageFromFile
#define HPDF_LoadPngImageFromFile2 vtklibharu_HPDF_LoadPngImageFromFile2
#define HPDF_LoadPngImageFromMem vtklibharu_HPDF_LoadPngImageFromMem
#define HPDF_LoadRawImageFromFile vtklibharu_HPDF_LoadRawImageFromFile
#define HPDF_LoadRawImageFromMem vtklibharu_HPDF_LoadRawImageFromMem
#define HPDF_LoadTTFontFromFile vtklibharu_HPDF_LoadTTFontFromFile
#define HPDF_LoadTTFontFromFile2 vtklibharu_HPDF_LoadTTFontFromFile2
#define HPDF_LoadType1FontFromFile vtklibharu_HPDF_LoadType1FontFromFile
#define HPDF_LoadU3DFromFile vtklibharu_HPDF_LoadU3DFromFile
#define HPDF_LoadU3DFromMem vtklibharu_HPDF_LoadU3DFromMem
#define HPDF_MarkupAnnot_New vtklibharu_HPDF_MarkupAnnot_New
#define HPDF_MarkupAnnot_SetCloudEffect vtklibharu_HPDF_MarkupAnnot_SetCloudEffect
#define HPDF_MarkupAnnot_SetCreationDate vtklibharu_HPDF_MarkupAnnot_SetCreationDate
#define HPDF_MarkupAnnot_SetIntent vtklibharu_HPDF_MarkupAnnot_SetIntent
#define HPDF_MarkupAnnot_SetInteriorCMYKColor vtklibharu_HPDF_MarkupAnnot_SetInteriorCMYKColor
#define HPDF_MarkupAnnot_SetInteriorGrayColor vtklibharu_HPDF_MarkupAnnot_SetInteriorGrayColor
#define HPDF_MarkupAnnot_SetInteriorRGBColor vtklibharu_HPDF_MarkupAnnot_SetInteriorRGBColor
#define HPDF_MarkupAnnot_SetInteriorTransparent vtklibharu_HPDF_MarkupAnnot_SetInteriorTransparent
#define HPDF_MarkupAnnot_SetPopup vtklibharu_HPDF_MarkupAnnot_SetPopup
#define HPDF_MarkupAnnot_SetRectDiff vtklibharu_HPDF_MarkupAnnot_SetRectDiff
#define HPDF_MarkupAnnot_SetSubject vtklibharu_HPDF_MarkupAnnot_SetSubject
#define HPDF_MarkupAnnot_SetTitle vtklibharu_HPDF_MarkupAnnot_SetTitle
#define HPDF_MarkupAnnot_SetTransparency vtklibharu_HPDF_MarkupAnnot_SetTransparency
#define HPDF_MD5Final vtklibharu_HPDF_MD5Final
#define HPDF_MD5Init vtklibharu_HPDF_MD5Init
#define HPDF_MD5Update vtklibharu_HPDF_MD5Update
#define HPDF_MemCmp vtklibharu_HPDF_MemCmp
#define HPDF_MemCpy vtklibharu_HPDF_MemCpy
#define HPDF_MemSet vtklibharu_HPDF_MemSet
#define HPDF_MemStream_FreeData vtklibharu_HPDF_MemStream_FreeData
#define HPDF_MemStream_FreeFunc vtklibharu_HPDF_MemStream_FreeFunc
#define HPDF_MemStream_GetBufCount vtklibharu_HPDF_MemStream_GetBufCount
#define HPDF_MemStream_GetBufPtr vtklibharu_HPDF_MemStream_GetBufPtr
#define HPDF_MemStream_GetBufSize vtklibharu_HPDF_MemStream_GetBufSize
#define HPDF_MemStream_InWrite vtklibharu_HPDF_MemStream_InWrite
#define HPDF_MemStream_New vtklibharu_HPDF_MemStream_New
#define HPDF_MemStream_ReadFunc vtklibharu_HPDF_MemStream_ReadFunc
#define HPDF_MemStream_Rewrite vtklibharu_HPDF_MemStream_Rewrite
#define HPDF_MemStream_SeekFunc vtklibharu_HPDF_MemStream_SeekFunc
#define HPDF_MemStream_SizeFunc vtklibharu_HPDF_MemStream_SizeFunc
#define HPDF_MemStream_TellFunc vtklibharu_HPDF_MemStream_TellFunc
#define HPDF_MemStream_WriteFunc vtklibharu_HPDF_MemStream_WriteFunc
#define HPDF_MMgr_Free vtklibharu_HPDF_MMgr_Free
#define HPDF_MMgr_New vtklibharu_HPDF_MMgr_New
#define HPDF_NameDict_GetNameTree vtklibharu_HPDF_NameDict_GetNameTree
#define HPDF_NameDict_New vtklibharu_HPDF_NameDict_New
#define HPDF_NameDict_SetNameTree vtklibharu_HPDF_NameDict_SetNameTree
#define HPDF_NameDict_Validate vtklibharu_HPDF_NameDict_Validate
#define HPDF_Name_GetValue vtklibharu_HPDF_Name_GetValue
#define HPDF_Name_New vtklibharu_HPDF_Name_New
#define HPDF_Name_SetValue vtklibharu_HPDF_Name_SetValue
#define HPDF_NameTree_Add vtklibharu_HPDF_NameTree_Add
#define HPDF_NameTree_New vtklibharu_HPDF_NameTree_New
#define HPDF_NameTree_Validate vtklibharu_HPDF_NameTree_Validate
#define HPDF_Name_Write vtklibharu_HPDF_Name_Write
#define HPDF_New vtklibharu_HPDF_New
#define HPDF_NewDoc vtklibharu_HPDF_NewDoc
#define HPDF_NewEx vtklibharu_HPDF_NewEx
#define HPDF_Null_New vtklibharu_HPDF_Null_New
#define HPDF_Number_New vtklibharu_HPDF_Number_New
#define HPDF_Number_SetValue vtklibharu_HPDF_Number_SetValue
#define HPDF_Number_Write vtklibharu_HPDF_Number_Write
#define HPDF_Obj_ForceFree vtklibharu_HPDF_Obj_ForceFree
#define HPDF_Obj_Free vtklibharu_HPDF_Obj_Free
#define HPDF_Obj_Write vtklibharu_HPDF_Obj_Write
#define HPDF_Obj_WriteValue vtklibharu_HPDF_Obj_WriteValue
#define HPDF_Outline_GetFirst vtklibharu_HPDF_Outline_GetFirst
#define HPDF_Outline_GetLast vtklibharu_HPDF_Outline_GetLast
#define HPDF_Outline_GetNext vtklibharu_HPDF_Outline_GetNext
#define HPDF_Outline_GetOpened vtklibharu_HPDF_Outline_GetOpened
#define HPDF_Outline_GetParent vtklibharu_HPDF_Outline_GetParent
#define HPDF_Outline_GetPrev vtklibharu_HPDF_Outline_GetPrev
#define HPDF_Outline_New vtklibharu_HPDF_Outline_New
#define HPDF_OutlineRoot_New vtklibharu_HPDF_OutlineRoot_New
#define HPDF_Outline_SetDestination vtklibharu_HPDF_Outline_SetDestination
#define HPDF_Outline_SetOpened vtklibharu_HPDF_Outline_SetOpened
#define HPDF_Outline_Validate vtklibharu_HPDF_Outline_Validate
#define HPDF_OutputIntent_New vtklibharu_HPDF_OutputIntent_New
#define HPDF_PadOrTrancatePasswd vtklibharu_HPDF_PadOrTrancatePasswd
#define HPDF_Page_Arc vtklibharu_HPDF_Page_Arc
#define HPDF_Page_BeginText vtklibharu_HPDF_Page_BeginText
#define HPDF_Page_CheckState vtklibharu_HPDF_Page_CheckState
#define HPDF_Page_Circle vtklibharu_HPDF_Page_Circle
#define HPDF_Page_Clip vtklibharu_HPDF_Page_Clip
#define HPDF_Page_ClosePath vtklibharu_HPDF_Page_ClosePath
#define HPDF_Page_ClosePathEofillStroke vtklibharu_HPDF_Page_ClosePathEofillStroke
#define HPDF_Page_ClosePathFillStroke vtklibharu_HPDF_Page_ClosePathFillStroke
#define HPDF_Page_ClosePathStroke vtklibharu_HPDF_Page_ClosePathStroke
#define HPDF_Page_Concat vtklibharu_HPDF_Page_Concat
#define HPDF_Page_Create3DAnnot vtklibharu_HPDF_Page_Create3DAnnot
#define HPDF_Page_Create3DAnnotExData vtklibharu_HPDF_Page_Create3DAnnotExData
#define HPDF_Page_Create3DC3DMeasure vtklibharu_HPDF_Page_Create3DC3DMeasure
#define HPDF_Page_Create3DView vtklibharu_HPDF_Page_Create3DView
#define HPDF_Page_CreateCircleAnnot vtklibharu_HPDF_Page_CreateCircleAnnot
#define HPDF_Page_CreateDestination vtklibharu_HPDF_Page_CreateDestination
#define HPDF_Page_CreateFreeTextAnnot vtklibharu_HPDF_Page_CreateFreeTextAnnot
#define HPDF_Page_CreateHighlightAnnot vtklibharu_HPDF_Page_CreateHighlightAnnot
#define HPDF_Page_CreateLineAnnot vtklibharu_HPDF_Page_CreateLineAnnot
#define HPDF_Page_CreateLinkAnnot vtklibharu_HPDF_Page_CreateLinkAnnot
#define HPDF_Page_CreatePD33DMeasure vtklibharu_HPDF_Page_CreatePD33DMeasure
#define HPDF_Page_CreatePopupAnnot vtklibharu_HPDF_Page_CreatePopupAnnot
#define HPDF_Page_CreateProjectionAnnot vtklibharu_HPDF_Page_CreateProjectionAnnot
#define HPDF_Page_CreateSquareAnnot vtklibharu_HPDF_Page_CreateSquareAnnot
#define HPDF_Page_CreateSquigglyAnnot vtklibharu_HPDF_Page_CreateSquigglyAnnot
#define HPDF_Page_CreateStampAnnot vtklibharu_HPDF_Page_CreateStampAnnot
#define HPDF_Page_CreateStrikeOutAnnot vtklibharu_HPDF_Page_CreateStrikeOutAnnot
#define HPDF_Page_CreateTextAnnot vtklibharu_HPDF_Page_CreateTextAnnot
#define HPDF_Page_CreateTextMarkupAnnot vtklibharu_HPDF_Page_CreateTextMarkupAnnot
#define HPDF_Page_CreateUnderlineAnnot vtklibharu_HPDF_Page_CreateUnderlineAnnot
#define HPDF_Page_CreateURILinkAnnot vtklibharu_HPDF_Page_CreateURILinkAnnot
#define HPDF_Page_CurveTo vtklibharu_HPDF_Page_CurveTo
#define HPDF_Page_CurveTo2 vtklibharu_HPDF_Page_CurveTo2
#define HPDF_Page_CurveTo3 vtklibharu_HPDF_Page_CurveTo3
#define HPDF_Page_DrawImage vtklibharu_HPDF_Page_DrawImage
#define HPDF_Page_Ellipse vtklibharu_HPDF_Page_Ellipse
#define HPDF_Page_EndPath vtklibharu_HPDF_Page_EndPath
#define HPDF_Page_EndText vtklibharu_HPDF_Page_EndText
#define HPDF_Page_Eoclip vtklibharu_HPDF_Page_Eoclip
#define HPDF_Page_Eofill vtklibharu_HPDF_Page_Eofill
#define HPDF_Page_EofillStroke vtklibharu_HPDF_Page_EofillStroke
#define HPDF_Page_ExecuteXObject vtklibharu_HPDF_Page_ExecuteXObject
#define HPDF_Page_Fill vtklibharu_HPDF_Page_Fill
#define HPDF_Page_FillStroke vtklibharu_HPDF_Page_FillStroke
#define HPDF_Page_GetCharSpace vtklibharu_HPDF_Page_GetCharSpace
#define HPDF_Page_GetCMYKFill vtklibharu_HPDF_Page_GetCMYKFill
#define HPDF_Page_GetCMYKStroke vtklibharu_HPDF_Page_GetCMYKStroke
#define HPDF_Page_GetCurrentFont vtklibharu_HPDF_Page_GetCurrentFont
#define HPDF_Page_GetCurrentFontSize vtklibharu_HPDF_Page_GetCurrentFontSize
#define HPDF_Page_GetCurrentPos vtklibharu_HPDF_Page_GetCurrentPos
#define HPDF_Page_GetCurrentPos2 vtklibharu_HPDF_Page_GetCurrentPos2
#define HPDF_Page_GetCurrentTextPos vtklibharu_HPDF_Page_GetCurrentTextPos
#define HPDF_Page_GetCurrentTextPos2 vtklibharu_HPDF_Page_GetCurrentTextPos2
#define HPDF_Page_GetDash vtklibharu_HPDF_Page_GetDash
#define HPDF_Page_GetExtGStateName vtklibharu_HPDF_Page_GetExtGStateName
#define HPDF_Page_GetFillingColorSpace vtklibharu_HPDF_Page_GetFillingColorSpace
#define HPDF_Page_GetFlat vtklibharu_HPDF_Page_GetFlat
#define HPDF_Page_GetGMode vtklibharu_HPDF_Page_GetGMode
#define HPDF_Page_GetGrayFill vtklibharu_HPDF_Page_GetGrayFill
#define HPDF_Page_GetGrayStroke vtklibharu_HPDF_Page_GetGrayStroke
#define HPDF_Page_GetGStateDepth vtklibharu_HPDF_Page_GetGStateDepth
#define HPDF_Page_GetHeight vtklibharu_HPDF_Page_GetHeight
#define HPDF_Page_GetHorizontalScalling vtklibharu_HPDF_Page_GetHorizontalScalling
#define HPDF_Page_GetInheritableItem vtklibharu_HPDF_Page_GetInheritableItem
#define HPDF_Page_GetLineCap vtklibharu_HPDF_Page_GetLineCap
#define HPDF_Page_GetLineJoin vtklibharu_HPDF_Page_GetLineJoin
#define HPDF_Page_GetLineWidth vtklibharu_HPDF_Page_GetLineWidth
#define HPDF_Page_GetLocalFontName vtklibharu_HPDF_Page_GetLocalFontName
#define HPDF_Page_GetMediaBox vtklibharu_HPDF_Page_GetMediaBox
#define HPDF_Page_GetMiterLimit vtklibharu_HPDF_Page_GetMiterLimit
#define HPDF_Page_GetRGBFill vtklibharu_HPDF_Page_GetRGBFill
#define HPDF_Page_GetRGBStroke vtklibharu_HPDF_Page_GetRGBStroke
#define HPDF_Page_GetShadingName vtklibharu_HPDF_Page_GetShadingName
#define HPDF_Page_GetStrokingColorSpace vtklibharu_HPDF_Page_GetStrokingColorSpace
#define HPDF_Page_GetTextLeading vtklibharu_HPDF_Page_GetTextLeading
#define HPDF_Page_GetTextMatrix vtklibharu_HPDF_Page_GetTextMatrix
#define HPDF_Page_GetTextRaise vtklibharu_HPDF_Page_GetTextRaise
#define HPDF_Page_GetTextRenderingMode vtklibharu_HPDF_Page_GetTextRenderingMode
#define HPDF_Page_GetTextRise vtklibharu_HPDF_Page_GetTextRise
#define HPDF_Page_GetTransMatrix vtklibharu_HPDF_Page_GetTransMatrix
#define HPDF_Page_GetWidth vtklibharu_HPDF_Page_GetWidth
#define HPDF_Page_GetWordSpace vtklibharu_HPDF_Page_GetWordSpace
#define HPDF_Page_GetXObjectName vtklibharu_HPDF_Page_GetXObjectName
#define HPDF_Page_GRestore vtklibharu_HPDF_Page_GRestore
#define HPDF_Page_GSave vtklibharu_HPDF_Page_GSave
#define HPDF_Page_InsertBefore vtklibharu_HPDF_Page_InsertBefore
#define HPDF_Page_Insert_Shared_Content_Stream vtklibharu_HPDF_Page_Insert_Shared_Content_Stream
#define HPDF_PageLabel_New vtklibharu_HPDF_PageLabel_New
#define HPDF_Page_LineTo vtklibharu_HPDF_Page_LineTo
#define HPDF_Page_MeasureText vtklibharu_HPDF_Page_MeasureText
#define HPDF_Page_MoveTextPos vtklibharu_HPDF_Page_MoveTextPos
#define HPDF_Page_MoveTextPos2 vtklibharu_HPDF_Page_MoveTextPos2
#define HPDF_Page_MoveTo vtklibharu_HPDF_Page_MoveTo
#define HPDF_Page_MoveToNextLine vtklibharu_HPDF_Page_MoveToNextLine
#define HPDF_Page_New vtklibharu_HPDF_Page_New
#define HPDF_Page_New_Content_Stream vtklibharu_HPDF_Page_New_Content_Stream
#define HPDF_Page_Rectangle vtklibharu_HPDF_Page_Rectangle
#define HPDF_Pages_AddKids vtklibharu_HPDF_Pages_AddKids
#define HPDF_Page_SetBoxValue vtklibharu_HPDF_Page_SetBoxValue
#define HPDF_Page_SetCharSpace vtklibharu_HPDF_Page_SetCharSpace
#define HPDF_Page_SetCMYKFill vtklibharu_HPDF_Page_SetCMYKFill
#define HPDF_Page_SetCMYKStroke vtklibharu_HPDF_Page_SetCMYKStroke
#define HPDF_Page_SetDash vtklibharu_HPDF_Page_SetDash
#define HPDF_Page_SetExtGState vtklibharu_HPDF_Page_SetExtGState
#define HPDF_Page_SetFilter vtklibharu_HPDF_Page_SetFilter
#define HPDF_Page_SetFlat vtklibharu_HPDF_Page_SetFlat
#define HPDF_Page_SetFontAndSize vtklibharu_HPDF_Page_SetFontAndSize
#define HPDF_Page_SetGrayFill vtklibharu_HPDF_Page_SetGrayFill
#define HPDF_Page_SetGrayStroke vtklibharu_HPDF_Page_SetGrayStroke
#define HPDF_Page_SetHeight vtklibharu_HPDF_Page_SetHeight
#define HPDF_Page_SetHorizontalScalling vtklibharu_HPDF_Page_SetHorizontalScalling
#define HPDF_Page_SetLineCap vtklibharu_HPDF_Page_SetLineCap
#define HPDF_Page_SetLineJoin vtklibharu_HPDF_Page_SetLineJoin
#define HPDF_Page_SetLineWidth vtklibharu_HPDF_Page_SetLineWidth
#define HPDF_Page_SetMiterLimit vtklibharu_HPDF_Page_SetMiterLimit
#define HPDF_Page_SetRGBFill vtklibharu_HPDF_Page_SetRGBFill
#define HPDF_Page_SetRGBStroke vtklibharu_HPDF_Page_SetRGBStroke
#define HPDF_Page_SetRotate vtklibharu_HPDF_Page_SetRotate
#define HPDF_Page_SetShading vtklibharu_HPDF_Page_SetShading
#define HPDF_Page_SetSize vtklibharu_HPDF_Page_SetSize
#define HPDF_Page_SetSlideShow vtklibharu_HPDF_Page_SetSlideShow
#define HPDF_Page_SetTextLeading vtklibharu_HPDF_Page_SetTextLeading
#define HPDF_Page_SetTextMatrix vtklibharu_HPDF_Page_SetTextMatrix
#define HPDF_Page_SetTextRaise vtklibharu_HPDF_Page_SetTextRaise
#define HPDF_Page_SetTextRenderingMode vtklibharu_HPDF_Page_SetTextRenderingMode
#define HPDF_Page_SetTextRise vtklibharu_HPDF_Page_SetTextRise
#define HPDF_Page_SetWidth vtklibharu_HPDF_Page_SetWidth
#define HPDF_Page_SetWordSpace vtklibharu_HPDF_Page_SetWordSpace
#define HPDF_Page_SetZoom vtklibharu_HPDF_Page_SetZoom
#define HPDF_Page_ShowText vtklibharu_HPDF_Page_ShowText
#define HPDF_Page_ShowTextNextLine vtklibharu_HPDF_Page_ShowTextNextLine
#define HPDF_Page_ShowTextNextLineEx vtklibharu_HPDF_Page_ShowTextNextLineEx
#define HPDF_Pages_New vtklibharu_HPDF_Pages_New
#define HPDF_Page_Stroke vtklibharu_HPDF_Page_Stroke
#define HPDF_Pages_Validate vtklibharu_HPDF_Pages_Validate
#define HPDF_Page_TextOut vtklibharu_HPDF_Page_TextOut
#define HPDF_Page_TextRect vtklibharu_HPDF_Page_TextRect
#define HPDF_Page_TextWidth vtklibharu_HPDF_Page_TextWidth
#define HPDF_Page_Validate vtklibharu_HPDF_Page_Validate
#define HPDF_PD33DMeasure_New vtklibharu_HPDF_PD33DMeasure_New
#define HPDF_PDFA_AppendOutputIntents vtklibharu_HPDF_PDFA_AppendOutputIntents
#define HPDF_PDFA_GenerateID vtklibharu_HPDF_PDFA_GenerateID
#define HPDF_PDFA_SetPDFAConformance vtklibharu_HPDF_PDFA_SetPDFAConformance
#define HPDF_PopupAnnot_New vtklibharu_HPDF_PopupAnnot_New
#define HPDF_PopupAnnot_SetOpened vtklibharu_HPDF_PopupAnnot_SetOpened
#define HPDF_ProjectionAnnot_New vtklibharu_HPDF_ProjectionAnnot_New
#define HPDF_ProjectionAnnot_SetExData vtklibharu_HPDF_ProjectionAnnot_SetExData
#define HPDF_Proxy_New vtklibharu_HPDF_Proxy_New
#define HPDF_RaiseError vtklibharu_HPDF_RaiseError
#define HPDF_ReadFromStream vtklibharu_HPDF_ReadFromStream
#define HPDF_Real_New vtklibharu_HPDF_Real_New
#define HPDF_Real_SetValue vtklibharu_HPDF_Real_SetValue
#define HPDF_Real_Write vtklibharu_HPDF_Real_Write
#define HPDF_ResetError vtklibharu_HPDF_ResetError
#define HPDF_ResetStream vtklibharu_HPDF_ResetStream
#define HPDF_SaveToFile vtklibharu_HPDF_SaveToFile
#define HPDF_SaveToStream vtklibharu_HPDF_SaveToStream
#define HPDF_SetCompressionMode vtklibharu_HPDF_SetCompressionMode
#define HPDF_SetCurrentEncoder vtklibharu_HPDF_SetCurrentEncoder
#define HPDF_SetEncryptionMode vtklibharu_HPDF_SetEncryptionMode
#define HPDF_SetError vtklibharu_HPDF_SetError
#define HPDF_SetErrorHandler vtklibharu_HPDF_SetErrorHandler
#define HPDF_SetInfoAttr vtklibharu_HPDF_SetInfoAttr
#define HPDF_SetInfoDateAttr vtklibharu_HPDF_SetInfoDateAttr
#define HPDF_SetOpenAction vtklibharu_HPDF_SetOpenAction
#define HPDF_SetPageLayout vtklibharu_HPDF_SetPageLayout
#define HPDF_SetPageMode vtklibharu_HPDF_SetPageMode
#define HPDF_SetPagesConfiguration vtklibharu_HPDF_SetPagesConfiguration
#define HPDF_SetPassword vtklibharu_HPDF_SetPassword
#define HPDF_SetPermission vtklibharu_HPDF_SetPermission
#define HPDF_SetViewerPreference vtklibharu_HPDF_SetViewerPreference
#define HPDF_Shading_AddVertexRGB vtklibharu_HPDF_Shading_AddVertexRGB
#define HPDF_Shading_New vtklibharu_HPDF_Shading_New
#define HPDF_StampAnnot_New vtklibharu_HPDF_StampAnnot_New
#define HPDF_StrCmp vtklibharu_HPDF_StrCmp
#define HPDF_StrCpy vtklibharu_HPDF_StrCpy
#define HPDF_Stream_CcittToStream vtklibharu_HPDF_Stream_CcittToStream
#define HPDF_Stream_Free vtklibharu_HPDF_Stream_Free
#define HPDF_Stream_Read vtklibharu_HPDF_Stream_Read
#define HPDF_Stream_ReadLn vtklibharu_HPDF_Stream_ReadLn
#define HPDF_Stream_Seek vtklibharu_HPDF_Stream_Seek
#define HPDF_Stream_Size vtklibharu_HPDF_Stream_Size
#define HPDF_Stream_Tell vtklibharu_HPDF_Stream_Tell
#define HPDF_Stream_Validate vtklibharu_HPDF_Stream_Validate
#define HPDF_Stream_Write vtklibharu_HPDF_Stream_Write
#define HPDF_Stream_WriteBinary vtklibharu_HPDF_Stream_WriteBinary
#define HPDF_Stream_WriteChar vtklibharu_HPDF_Stream_WriteChar
#define HPDF_Stream_WriteEscapeName vtklibharu_HPDF_Stream_WriteEscapeName
#define HPDF_Stream_WriteEscapeText vtklibharu_HPDF_Stream_WriteEscapeText
#define HPDF_Stream_WriteEscapeText2 vtklibharu_HPDF_Stream_WriteEscapeText2
#define HPDF_Stream_WriteInt vtklibharu_HPDF_Stream_WriteInt
#define HPDF_Stream_WriteReal vtklibharu_HPDF_Stream_WriteReal
#define HPDF_Stream_WriteStr vtklibharu_HPDF_Stream_WriteStr
#define HPDF_Stream_WriteToStream vtklibharu_HPDF_Stream_WriteToStream
#define HPDF_Stream_WriteToStreamWithDeflate vtklibharu_HPDF_Stream_WriteToStreamWithDeflate
#define HPDF_Stream_WriteUChar vtklibharu_HPDF_Stream_WriteUChar
#define HPDF_Stream_WriteUInt vtklibharu_HPDF_Stream_WriteUInt
#define HPDF_String_Cmp vtklibharu_HPDF_String_Cmp
#define HPDF_String_Free vtklibharu_HPDF_String_Free
#define HPDF_String_New vtklibharu_HPDF_String_New
#define HPDF_String_SetValue vtklibharu_HPDF_String_SetValue
#define HPDF_String_Write vtklibharu_HPDF_String_Write
#define HPDF_StrLen vtklibharu_HPDF_StrLen
#define HPDF_StrStr vtklibharu_HPDF_StrStr
#define HPDF_TextAnnot_SetIcon vtklibharu_HPDF_TextAnnot_SetIcon
#define HPDF_TextAnnot_SetOpened vtklibharu_HPDF_TextAnnot_SetOpened
#define HPDF_TextMarkupAnnot_SetQuadPoints vtklibharu_HPDF_TextMarkupAnnot_SetQuadPoints
#define HPDF_TIFFFaxBlackCodes vtklibharu_HPDF_TIFFFaxBlackCodes
#define HPDF_TIFFFaxWhiteCodes vtklibharu_HPDF_TIFFFaxWhiteCodes
#define HPDF_ToBox vtklibharu_HPDF_ToBox
#define HPDF_ToPoint vtklibharu_HPDF_ToPoint
#define HPDF_ToRect vtklibharu_HPDF_ToRect
#define HPDF_TTFontDef_GetCharBBox vtklibharu_HPDF_TTFontDef_GetCharBBox
#define HPDF_TTFontDef_GetCharWidth vtklibharu_HPDF_TTFontDef_GetCharWidth
#define HPDF_TTFontDef_GetGidWidth vtklibharu_HPDF_TTFontDef_GetGidWidth
#define HPDF_TTFontDef_GetGlyphid vtklibharu_HPDF_TTFontDef_GetGlyphid
#define HPDF_TTFontDef_Load vtklibharu_HPDF_TTFontDef_Load
#define HPDF_TTFontDef_Load2 vtklibharu_HPDF_TTFontDef_Load2
#define HPDF_TTFontDef_New vtklibharu_HPDF_TTFontDef_New
#define HPDF_TTFontDef_SaveFontData vtklibharu_HPDF_TTFontDef_SaveFontData
#define HPDF_TTFontDef_SetTagName vtklibharu_HPDF_TTFontDef_SetTagName
#define HPDF_TTFont_New vtklibharu_HPDF_TTFont_New
#define HPDF_Type0Font_New vtklibharu_HPDF_Type0Font_New
#define HPDF_Type1FontDef_Duplicate vtklibharu_HPDF_Type1FontDef_Duplicate
#define HPDF_Type1FontDef_GetWidth vtklibharu_HPDF_Type1FontDef_GetWidth
#define HPDF_Type1FontDef_GetWidthByName vtklibharu_HPDF_Type1FontDef_GetWidthByName
#define HPDF_Type1FontDef_Load vtklibharu_HPDF_Type1FontDef_Load
#define HPDF_Type1FontDef_New vtklibharu_HPDF_Type1FontDef_New
#define HPDF_Type1FontDef_SetWidths vtklibharu_HPDF_Type1FontDef_SetWidths
#define HPDF_Type1Font_New vtklibharu_HPDF_Type1Font_New
#define HPDF_U3D_Add3DView vtklibharu_HPDF_U3D_Add3DView
#define HPDF_U3D_AddOnInstanciate vtklibharu_HPDF_U3D_AddOnInstanciate
#define HPDF_U3D_LoadU3D vtklibharu_HPDF_U3D_LoadU3D
#define HPDF_U3D_LoadU3DFromMem vtklibharu_HPDF_U3D_LoadU3DFromMem
#define HPDF_U3D_SetDefault3DView vtklibharu_HPDF_U3D_SetDefault3DView
#define HPDF_UInt16Swap vtklibharu_HPDF_UInt16Swap
#define HPDF_UnicodeToGryphName vtklibharu_HPDF_UnicodeToGryphName
#define HPDF_URILinkAnnot_New vtklibharu_HPDF_URILinkAnnot_New
#define HPDF_UseCNSEncodings vtklibharu_HPDF_UseCNSEncodings
#define HPDF_UseCNSFonts vtklibharu_HPDF_UseCNSFonts
#define HPDF_UseCNTEncodings vtklibharu_HPDF_UseCNTEncodings
#define HPDF_UseCNTFonts vtklibharu_HPDF_UseCNTFonts
#define HPDF_UseJPEncodings vtklibharu_HPDF_UseJPEncodings
#define HPDF_UseJPFonts vtklibharu_HPDF_UseJPFonts
#define HPDF_UseKREncodings vtklibharu_HPDF_UseKREncodings
#define HPDF_UseKRFonts vtklibharu_HPDF_UseKRFonts
#define HPDF_UseUTFEncodings vtklibharu_HPDF_UseUTFEncodings
#define HPDF_Xref_Add vtklibharu_HPDF_Xref_Add
#define HPDF_Xref_Free vtklibharu_HPDF_Xref_Free
#define HPDF_Xref_GetEntry vtklibharu_HPDF_Xref_GetEntry
#define HPDF_Xref_GetEntryByObjectId vtklibharu_HPDF_Xref_GetEntryByObjectId
#define HPDF_Xref_New vtklibharu_HPDF_Xref_New
#define HPDF_Xref_WriteToStream vtklibharu_HPDF_Xref_WriteToStream

#endif
